@@ -48,12 +48,11 @@ GetOptions(\%args, qw(
     build=s host=s big-endian jit! enable-jit
     prefix=s bindir=s libdir=s mastdir=s
     relocatable make-install asan ubsan tsan
-    valgrind telemeh dtrace show-autovect git-cache-dir=s
-    show-autovect-failed:s),
+    valgrind telemeh! dtrace show-autovect git-cache-dir=s
+    show-autovect-failed:s mimalloc! has-mimalloc c11-atomics!),
 
     'no-optimize|nooptimize' => sub { $args{optimize} = 0 },
     'no-debug|nodebug' => sub { $args{debug} = 0 },
-    'no-telemeh|notelemeh' => sub { $args{telemeh} = 0 }
 ) or die "See --help for further information\n";
 
 
@@ -92,7 +91,7 @@ if ( $args{relocatable} && ($^O eq 'aix' || $^O eq 'openbsd') ) {
 }
 
 for (qw(coverage static big-endian has-libtommath has-sha has-libuv
-        has-libatomic_ops asan ubsan tsan valgrind dtrace show-vec)) {
+        has-libatomic_ops has-mimalloc asan ubsan tsan valgrind dtrace show-vec)) {
     $args{$_} = 0 unless defined $args{$_};
 }
 
@@ -273,7 +272,24 @@ else {
     push @hllincludes, 'libuv';
 }
 
-if ($args{'has-libatomic_ops'}) {
+# we just need a minimal ldlibs configured for the stdatomic probe,
+# it will get overwritten later
+$config{ldlibs} = join ' ',
+    $config{lincludes},
+    (map { sprintf $config{ldusr}, $_; } @{$config{usrlibs}}),
+    (map { sprintf $config{ldsys}, $_; } @{$config{syslibs}});
+
+# probe for working stdatomic.h, also used by mimalloc
+build::probe::stdatomic(\%config, \%defaults);
+
+$config{use_c11_atomics} = defined $args{'c11-atomics'}
+    ? $args{'c11-atomics'}   ? 1 : 0
+    : $config{has_stdatomic} ? 1 : 0; # default to on if available
+
+if ($config{use_c11_atomics}) {
+    $defaults{-thirdparty}->{lao} = undef;
+}
+elsif ($args{'has-libatomic_ops'}) {
     $defaults{-thirdparty}->{lao} = undef;
     unshift @{$config{usrlibs}}, 'atomic_ops';
     setup_native_library('atomic_ops') if $config{pkgconfig_works};
@@ -381,6 +397,54 @@ else {
     $config{heapsnapformat} = 2;
 }
 
+$config{use_mimalloc} = $args{mimalloc};
+if (!defined $config{use_mimalloc}) {
+    if ($config{has_stdatomic}) {
+        print "Defaulting to mimalloc because you have <stdatomic.h>\n";
+        $config{use_mimalloc} = 1;
+    }
+    elsif ($config{cc} eq 'cl') {
+        print "Defaulting to mimalloc because you are using MSVC\n";
+        $config{use_mimalloc} = 1;
+    }
+    else {
+        print "Defaulting to libc malloc because <stdatomic.h> was not found.\n";
+        $config{use_mimalloc} = 0;
+    }
+}
+
+if ($config{use_mimalloc}) {
+    $config{cflags} .= ' -DMI_SKIP_COLLECT_ON_EXIT';
+    if ($args{'has-mimalloc'}) {
+        $config{mimalloc_include} = "";
+        $config{mimalloc_object} = "";
+        $defaults{-thirdparty}->{mimalloc} = undef;
+        unshift @{$config{usrlibs}}, 'mimalloc';
+        setup_native_library('mimalloc') if $config{pkgconfig_works};
+        if (not $config{crossconf}) {
+            if (index($config{cincludes}, '-I/usr/local/include') == -1) {
+                $config{cincludes} = join(' ', $config{cincludes}, '-I/usr/local/include');
+            }
+            if (index($config{lincludes}, '-L/usr/local/lib') == -1) {
+                $config{lincludes} = join(' ', $config{lincludes}, '-L/usr/local/lib');
+            }
+        }
+    }
+    else {
+        $config{moar_cincludes} .= ' ' . $defaults{ccinc} . '3rdparty/mimalloc/include'
+                                 . ' ' . $defaults{ccinc} . '3rdparty/mimalloc/src';
+        $config{install}   .= "\t\$(MKPATH) \"\$(DESTDIR)\$(PREFIX)/include/mimalloc\"\n"
+                           . "\t\$(CP) 3rdparty/mimalloc/include/*.h \"\$(DESTDIR)\$(PREFIX)/include/mimalloc\"\n";
+        push @hllincludes, 'mimalloc';
+        $config{mimalloc_include} = '@ccincsystem@3rdparty/mimalloc';
+        $config{mimalloc_object} = '3rdparty/mimalloc/src/static@obj@';
+    }
+}
+else {
+    $config{mimalloc_include} = "";
+    $config{mimalloc_object} = "";
+}
+
 # mangle library names
 $config{ldlibs} = join ' ',
     $config{lincludes},
@@ -401,6 +465,7 @@ $config{lddebugflags} = sprintf $config{lddebugflags}, defined_or $args{debug},
 
 # generate CFLAGS
 my @cflags;
+push @cflags, '-std=c99' if $defaults{os} eq 'mingw32';
 push @cflags, $config{ccmiscflags};
 push @cflags, $config{ccoptiflags}  if $args{optimize};
 push @cflags, $config{ccdebugflags} if $args{debug};
@@ -408,6 +473,7 @@ push @cflags, $config{cc_covflags}  if $args{coverage};
 push @cflags, $config{ccwarnflags};
 push @cflags, $config{ccdefflags};
 push @cflags, $config{ccshared}     unless $args{static};
+push @cflags, '-gdwarf-4'           if $config{cc} eq 'clang';
 push @cflags,
 $config{cc} eq 'clang'
     ? '-Rpass=loop-vectorize'
@@ -1084,6 +1150,27 @@ default.
 Toggle optimization and debug flags during compile and link. If nothing
 is specified the default is to optimize.
 
+=item --mimalloc
+
+=item --no-mimalloc
+
+Control whether we build with mimalloc or use the C library's malloc.
+
+mimalloc requires either MSVC or working C11 C<< <stdatomic> >>. We probe for
+this, and if found or using MSVC we default to use mimalloc. Otherwise we
+default to the C library's malloc. Specify C<--no-mimalloc> to force use of the
+C library's malloc always. Specify C<--mimalloc> to force use of mimalloc, even
+if the probing thinks that it won't build.
+
+=item --c11-atomics
+
+=item --no-c11-atomics
+
+Use C11 atomics instead of libatomic_ops for atomic operations. The default
+is currently C<--no-c11-atomics> - ie use libatomic_ops. If you set
+C<--c11-atomics> and your compiler does not support C11 atomics, your build
+will fail.
+
 =item --os <os>
 
 Set the operating system name which you are compiling to.
@@ -1229,6 +1316,8 @@ Build and install MoarVM in addition to configuring it.
 
 =item --has-libffi
 
+=item --has-mimalloc
+
 =item --pkgconfig=/path/to/pkgconfig/executable
 
 Provide path to the pkgconfig executable. Default: /usr/bin/pkg-config
